@@ -1,6 +1,6 @@
 /* 
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2009, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2010, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -35,9 +35,14 @@
 #include <curl/curl.h>
 #define MAX_URLS 20
 
+#define ENCODING_NONE 0
+#define ENCODING_DEFAULT 1
+#define ENCODING_BASE64 2
+#define ENCODING_TEXTXML 2
+
 static struct {
 	char *cred;
-	char *urls[MAX_URLS+1];
+	char *urls[MAX_URLS + 1];
 	int url_count;
 	int url_index;
 	switch_thread_rwlock_t *log_path_lock;
@@ -56,10 +61,12 @@ static struct {
 	char *ssl_cacert_file;
 	uint32_t enable_ssl_verifyhost;
 	int encode;
+	int log_http_and_disk;
 	int log_b;
 	int prefix_a;
 	int disable100continue;
 	int rotate;
+	int auth_scheme;
 	switch_memory_pool_t *pool;
 } globals;
 
@@ -89,7 +96,7 @@ static switch_status_t set_xml_cdr_log_dirs()
 
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Rotating log file paths\n");
 
-	if (!switch_strlen_zero(globals.base_log_dir)) {
+	if (!zstr(globals.base_log_dir)) {
 		if (globals.rotate) {
 			if ((path = switch_mprintf("%s%s%s", globals.base_log_dir, SWITCH_PATH_SEPARATOR, date))) {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Rotating log file path to %s\n", path);
@@ -104,34 +111,30 @@ static switch_status_t set_xml_cdr_log_dirs()
 					switch_safe_free(globals.log_dir);
 					globals.log_dir = path;
 					switch_thread_rwlock_unlock(globals.log_path_lock);
-				}
-				else {
+				} else {
 					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Failed to create new mod_xml_cdr log_dir path\n");
 					switch_safe_free(path);
 					status = SWITCH_STATUS_FALSE;
 				}
-			}
-			else {
+			} else {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Failed to generate new mod_xml_cdr log_dir path\n");
 				status = SWITCH_STATUS_FALSE;
 			}
-		}
-		else {
+		} else {
 			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Setting log file path to %s\n", globals.base_log_dir);
 			if ((path = switch_safe_strdup(globals.base_log_dir))) {
 				switch_thread_rwlock_wrlock(globals.log_path_lock);
 				switch_safe_free(globals.log_dir);
 				globals.log_dir = path;
 				switch_thread_rwlock_unlock(globals.log_path_lock);
-			}
-			else {
+			} else {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Failed to set log_dir path\n");
 				status = SWITCH_STATUS_FALSE;
 			}
 		}
 	}
 
-	if (!switch_strlen_zero(globals.base_err_log_dir)) {
+	if (!zstr(globals.base_err_log_dir)) {
 		if (globals.rotate) {
 			if ((path = switch_mprintf("%s%s%s", globals.base_err_log_dir, SWITCH_PATH_SEPARATOR, date))) {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Rotating err log file path to %s\n", path);
@@ -146,27 +149,23 @@ static switch_status_t set_xml_cdr_log_dirs()
 					switch_safe_free(globals.err_log_dir);
 					globals.err_log_dir = path;
 					switch_thread_rwlock_unlock(globals.log_path_lock);
-				}
-				else {
+				} else {
 					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Failed to create new mod_xml_cdr err_log_dir path\n");
 					switch_safe_free(path);
 					status = SWITCH_STATUS_FALSE;
 				}
-			}
-			else {
+			} else {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Failed to generate new mod_xml_cdr err_log_dir path\n");
 				status = SWITCH_STATUS_FALSE;
 			}
-		}
-		else {
+		} else {
 			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Setting err log file path to %s\n", globals.base_err_log_dir);
 			if ((path = switch_safe_strdup(globals.base_err_log_dir))) {
 				switch_thread_rwlock_wrlock(globals.log_path_lock);
 				switch_safe_free(globals.err_log_dir);
 				globals.err_log_dir = path;
 				switch_thread_rwlock_unlock(globals.log_path_lock);
-			}
-			else {
+			} else {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Failed to set err_log_dir path\n");
 				status = SWITCH_STATUS_FALSE;
 			}
@@ -193,7 +192,7 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 	switch_channel_t *channel = switch_core_session_get_channel(session);
 	switch_status_t status = SWITCH_STATUS_FALSE;
 	int is_b;
-	const char * a_prefix = "";
+	const char *a_prefix = "";
 
 	if (globals.shutdown) {
 		return SWITCH_STATUS_SUCCESS;
@@ -203,7 +202,8 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 	if (!globals.log_b && is_b) {
 		return SWITCH_STATUS_SUCCESS;
 	}
-	if (!is_b && globals.prefix_a) a_prefix = "a_";
+	if (!is_b && globals.prefix_a)
+		a_prefix = "a_";
 
 	if (switch_ivr_generate_xml_cdr(session, &cdr) != SWITCH_STATUS_SUCCESS) {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error Generating Data!\n");
@@ -223,14 +223,14 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 		logdir = globals.log_dir;
 	}
 
-	if (!switch_strlen_zero(logdir)) {
+	if (!zstr(logdir) && (globals.log_http_and_disk || !globals.url_count)) {
 		path = switch_mprintf("%s%s%s%s.cdr.xml", logdir, SWITCH_PATH_SEPARATOR, a_prefix, switch_core_session_get_uuid(session));
 		switch_thread_rwlock_unlock(globals.log_path_lock);
 		if (path) {
 #ifdef _MSC_VER
 			if ((fd = open(path, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR)) > -1) {
 #else
-			if ((fd = open(path, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR| S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)) > -1) {
+			if ((fd = open(path, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)) > -1) {
 #endif
 				int wrote;
 				wrote = write(fd, xml_text, (unsigned) strlen(xml_text));
@@ -247,22 +247,24 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 			}
 			switch_safe_free(path);
 		}
-	}
-	else {
+	} else {
 		switch_thread_rwlock_unlock(globals.log_path_lock);
 	}
 
 	/* try to post it to the web server */
 	if (globals.url_count) {
+		char *destUrl = NULL;
 		curl_handle = curl_easy_init();
 
-		if (globals.encode) {
+		if (globals.encode == ENCODING_TEXTXML) {
+			headers = curl_slist_append(headers, "Content-Type: text/xml");
+		} else if (globals.encode) {
 			switch_size_t need_bytes = strlen(xml_text) * 3;
 
 			xml_text_escaped = malloc(need_bytes);
 			switch_assert(xml_text_escaped);
 			memset(xml_text_escaped, 0, need_bytes);
-			if (globals.encode == 1) {
+			if (globals.encode == ENCODING_DEFAULT) {
 				headers = curl_slist_append(headers, "Content-Type: application/x-www-form-urlencoded");
 				switch_url_encode(xml_text, xml_text_escaped, need_bytes);
 			} else {
@@ -275,13 +277,15 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 			headers = curl_slist_append(headers, "Content-Type: application/x-www-form-plaintext");
 		}
 
-		if (!(curl_xml_text = switch_mprintf("cdr=%s", xml_text))) {
+		if (globals.encode == ENCODING_TEXTXML) {
+			curl_xml_text = xml_text;
+		} else if (!(curl_xml_text = switch_mprintf("cdr=%s", xml_text))) {
 			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Memory Error!\n");
 			goto error;
 		}
 
-		if (!switch_strlen_zero(globals.cred)) {
-			curl_easy_setopt(curl_handle, CURLOPT_HTTPAUTH, CURLAUTH_ANY);
+		if (!zstr(globals.cred)) {
+			curl_easy_setopt(curl_handle, CURLOPT_HTTPAUTH, globals.auth_scheme);
 			curl_easy_setopt(curl_handle, CURLOPT_USERPWD, globals.cred);
 		}
 
@@ -295,19 +299,19 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 			slist = curl_slist_append(slist, "Expect:");
 			curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, slist);
 		}
-		
+
 		if (globals.ssl_cert_file) {
 			curl_easy_setopt(curl_handle, CURLOPT_SSLCERT, globals.ssl_cert_file);
 		}
-		
+
 		if (globals.ssl_key_file) {
 			curl_easy_setopt(curl_handle, CURLOPT_SSLKEY, globals.ssl_key_file);
 		}
-		
+
 		if (globals.ssl_key_password) {
 			curl_easy_setopt(curl_handle, CURLOPT_SSLKEYPASSWD, globals.ssl_key_password);
 		}
-		
+
 		if (globals.ssl_version) {
 			if (!strcasecmp(globals.ssl_version, "SSLv3")) {
 				curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv3);
@@ -315,7 +319,7 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 				curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
 			}
 		}
-		
+
 		if (globals.ssl_cacert_file) {
 			curl_easy_setopt(curl_handle, CURLOPT_CAINFO, globals.ssl_cacert_file);
 		}
@@ -329,28 +333,30 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 			if (cur_try > 0) {
 				switch_yield(globals.delay * 1000000);
 			}
-			
-			curl_easy_setopt(curl_handle, CURLOPT_URL, globals.urls[globals.url_index]);
 
-			if (!strncasecmp(globals.urls[globals.url_index], "https", 5)) {
+			destUrl = switch_mprintf("%s?uuid=%s", globals.urls[globals.url_index], switch_core_session_get_uuid(session));
+			curl_easy_setopt(curl_handle, CURLOPT_URL, destUrl);
+
+			if (!strncasecmp(destUrl, "https", 5)) {
 				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
 				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
 			}
-			
+
 			if (globals.enable_cacert_check) {
 				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, TRUE);
 			}
-			
+
 			if (globals.enable_ssl_verifyhost) {
 				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 2);
 			}
 
 			curl_easy_perform(curl_handle);
 			curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &httpRes);
+			switch_safe_free(destUrl);
 			if (httpRes == 200) {
 				goto success;
 			} else {
-				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Got error [%ld] posting to web server [%s]\n", 
+				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Got error [%ld] posting to web server [%s]\n",
 								  httpRes, globals.urls[globals.url_index]);
 				globals.url_index++;
 				switch_assert(globals.url_count <= MAX_URLS);
@@ -408,7 +414,9 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 	if (slist) {
 		curl_slist_free_all(slist);
 	}
-	switch_safe_free(curl_xml_text);
+	if (curl_xml_text != xml_text) {
+		switch_safe_free(curl_xml_text);
+	}
 	switch_safe_free(xml_text);
 	switch_safe_free(path);
 	switch_xml_free(cdr);
@@ -434,11 +442,11 @@ static switch_state_handler_table_t state_handlers = {
 	/*.on_hangup */ NULL,
 	/*.on_exchange_media */ NULL,
 	/*.on_soft_execute */ NULL,
-	/*.on_consume_media*/ NULL,
-	/*.on_hibernate*/ NULL,
-	/*.on_reset*/ NULL,
-	/*.on_park*/ NULL,
-	/*.on_reporting*/ my_on_reporting
+	/*.on_consume_media */ NULL,
+	/*.on_hibernate */ NULL,
+	/*.on_reset */ NULL,
+	/*.on_park */ NULL,
+	/*.on_reporting */ my_on_reporting
 };
 
 SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
@@ -458,9 +466,12 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 	}
 
 	memset(&globals, 0, sizeof(globals));
+	globals.log_http_and_disk = 0;
 	globals.log_b = 1;
 	globals.disable100continue = 0;
 	globals.pool = pool;
+	globals.auth_scheme = CURLAUTH_BASIC;
+
 	switch_thread_rwlock_create(&globals.log_path_lock, pool);
 
 	/* parse the config */
@@ -474,15 +485,17 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 			char *var = (char *) switch_xml_attr_soft(param, "name");
 			char *val = (char *) switch_xml_attr_soft(param, "value");
 
-			if (!strcasecmp(var, "cred") && !switch_strlen_zero(val)) {
+			if (!strcasecmp(var, "cred") && !zstr(val)) {
 				globals.cred = switch_core_strdup(globals.pool, val);
-			} else if (!strcasecmp(var, "url") && !switch_strlen_zero(val)) {
+			} else if (!strcasecmp(var, "url") && !zstr(val)) {
 				if (globals.url_count >= MAX_URLS) {
 					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "maximum urls configured!\n");
 				} else {
 					globals.urls[globals.url_count++] = switch_core_strdup(globals.pool, val);
 				}
-			} else if (!strcasecmp(var, "delay") && !switch_strlen_zero(val)) {
+			} else if (!strcasecmp(var, "log-http-and-disk")) {
+				globals.log_http_and_disk = switch_true(val);
+			} else if (!strcasecmp(var, "delay") && !zstr(val)) {
 				globals.delay = (uint32_t) atoi(val);
 			} else if (!strcasecmp(var, "log-b-leg")) {
 				globals.log_b = switch_true(val);
@@ -490,18 +503,20 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 				globals.prefix_a = switch_true(val);
 			} else if (!strcasecmp(var, "disable-100-continue") && switch_true(val)) {
 				globals.disable100continue = 1;
-			} else if (!strcasecmp(var, "encode") && !switch_strlen_zero(val)) {
+			} else if (!strcasecmp(var, "encode") && !zstr(val)) {
 				if (!strcasecmp(val, "base64")) {
-					globals.encode = 2;
+					globals.encode = ENCODING_BASE64;
+				} else if (!strcasecmp(val, "textxml")) {
+					globals.encode = ENCODING_TEXTXML;
 				} else {
-					globals.encode = switch_true(val) ? 1 : 0;
+					globals.encode = switch_true(val) ? ENCODING_DEFAULT : ENCODING_NONE;
 				}
-			} else if (!strcasecmp(var, "retries") && !switch_strlen_zero(val)) {
+			} else if (!strcasecmp(var, "retries") && !zstr(val)) {
 				globals.retries = (uint32_t) atoi(val);
-			} else if (!strcasecmp(var, "rotate") && !switch_strlen_zero(val)) {
+			} else if (!strcasecmp(var, "rotate") && !zstr(val)) {
 				globals.rotate = switch_true(val);
 			} else if (!strcasecmp(var, "log-dir")) {
-				if (switch_strlen_zero(val)) {
+				if (zstr(val)) {
 					globals.base_log_dir = switch_mprintf("%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
 				} else {
 					if (switch_is_file_path(val)) {
@@ -511,7 +526,7 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 					}
 				}
 			} else if (!strcasecmp(var, "err-log-dir")) {
-				if (switch_strlen_zero(val)) {
+				if (zstr(val)) {
 					globals.base_err_log_dir = switch_mprintf("%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
 				} else {
 					if (switch_is_file_path(val)) {
@@ -534,10 +549,27 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 				globals.ssl_cacert_file = val;
 			} else if (!strcasecmp(var, "enable-ssl-verifyhost") && switch_true(val)) {
 				globals.enable_ssl_verifyhost = 1;
+			} else if (!strcasecmp(var, "auth-scheme")) {
+				if (*val == '=') {
+					globals.auth_scheme = 0;
+					val++;
+				}
+
+				if (!strcasecmp(val, "basic")) {
+					globals.auth_scheme |= CURLAUTH_BASIC;
+				} else if (!strcasecmp(val, "digest")) {
+					globals.auth_scheme |= CURLAUTH_DIGEST;
+				} else if (!strcasecmp(val, "NTLM")) {
+					globals.auth_scheme |= CURLAUTH_NTLM;
+				} else if (!strcasecmp(val, "GSS-NEGOTIATE")) {
+					globals.auth_scheme |= CURLAUTH_GSSNEGOTIATE;
+				} else if (!strcasecmp(val, "any")) {
+					globals.auth_scheme = CURLAUTH_ANY;
+				}
 			}
-		
-			if (switch_strlen_zero(globals.base_err_log_dir)) {
-				if (!switch_strlen_zero(globals.base_log_dir)) {
+
+			if (zstr(globals.base_err_log_dir)) {
+				if (!zstr(globals.base_log_dir)) {
 					globals.base_err_log_dir = switch_core_strdup(globals.pool, globals.base_log_dir);
 				} else {
 					globals.base_err_log_dir = switch_core_sprintf(globals.pool, "%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
@@ -567,7 +599,7 @@ SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_xml_cdr_shutdown)
 {
 
 	globals.shutdown = 1;
-	
+
 	switch_core_remove_state_handler(&state_handlers);
 	return SWITCH_STATUS_SUCCESS;
 }
