@@ -1,6 +1,6 @@
 /* 
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2009, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2010, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -52,7 +52,7 @@
 SWITCH_DECLARE_DATA switch_directories SWITCH_GLOBAL_dirs = { 0 };
 
 /* The main runtime obj we keep this hidden for ourselves */
-struct switch_runtime runtime;
+struct switch_runtime runtime = { 0 };
 static void switch_load_core_config(const char *file);
 
 static void send_heartbeat(void)
@@ -82,6 +82,7 @@ static void send_heartbeat(void)
 		switch_event_add_header(event, SWITCH_STACK_BOTTOM, "Session-Count", "%u", switch_core_session_count());
 		switch_event_add_header(event, SWITCH_STACK_BOTTOM, "Session-Per-Sec", "%u", runtime.sps);
 		switch_event_add_header(event, SWITCH_STACK_BOTTOM, "Session-Since-Startup", "%" SWITCH_SIZE_T_FMT, switch_core_session_id() - 1);
+		switch_event_add_header(event, SWITCH_STACK_BOTTOM, "Idle-CPU", "%f", switch_core_idle_cpu());
 		switch_event_fire(&event);
 	}
 }
@@ -89,13 +90,18 @@ static void send_heartbeat(void)
 static char main_ip4[256] = "";
 static char main_ip6[256] = "";
 
-static void check_ip(void) {
+static void check_ip(void)
+{
 	char guess_ip4[256] = "";
 	char guess_ip6[256] = "";
 	char old_ip4[256] = "";
 	char old_ip6[256] = "";
 	int ok4 = 1, ok6 = 1;
 	int mask = 0;
+	static char hostname[256] = "";
+
+	gethostname(hostname, sizeof(hostname));
+	switch_core_set_variable("hostname", hostname);
 
 	switch_find_local_ip(guess_ip4, sizeof(guess_ip4), &mask, AF_INET);
 	switch_find_local_ip(guess_ip6, sizeof(guess_ip6), NULL, AF_INET6);
@@ -123,7 +129,7 @@ static void check_ip(void) {
 			switch_core_set_variable("local_ip_v6", guess_ip6);
 		}
 	}
-	
+
 	if (!ok4 || !ok6) {
 		switch_event_t *event;
 
@@ -187,8 +193,8 @@ SWITCH_DECLARE(FILE *) switch_core_data_channel(switch_text_channel_t channel)
 SWITCH_DECLARE(void) switch_core_remove_state_handler(const switch_state_handler_table_t *state_handler)
 {
 	int index, tmp_index = 0;
-	const switch_state_handler_table_t *tmp[SWITCH_MAX_STATE_HANDLERS+1] = { 0 };
-	
+	const switch_state_handler_table_t *tmp[SWITCH_MAX_STATE_HANDLERS + 1] = { 0 };
+
 	switch_mutex_lock(runtime.global_mutex);
 
 	for (index = 0; index < runtime.state_handler_index; index++) {
@@ -264,8 +270,8 @@ SWITCH_DECLARE(char *) switch_core_get_variable(const char *varname)
 static void switch_core_unset_variables(void)
 {
 	switch_hash_index_t *hi;
-    const void *var;
-    void *val;
+	const void *var;
+	void *val;
 
 	switch_mutex_lock(runtime.global_var_mutex);
 	for (hi = switch_hash_first(NULL, runtime.global_vars); hi; hi = switch_hash_next(hi)) {
@@ -302,19 +308,21 @@ SWITCH_DECLARE(char *) switch_core_get_uuid(void)
 }
 
 
-static void *switch_core_service_thread(switch_thread_t *thread, void *obj)
+static void *SWITCH_THREAD_FUNC switch_core_service_thread(switch_thread_t *thread, void *obj)
 {
 	switch_core_session_t *session = obj;
 	switch_channel_t *channel;
 	switch_frame_t *read_frame;
 
-//	switch_assert(thread != NULL);
-//	switch_assert(session != NULL);
+//  switch_assert(thread != NULL);
+//  switch_assert(session != NULL);
 
 	if (switch_core_session_read_lock(session) != SWITCH_STATUS_SUCCESS) {
 		return NULL;
 	}
 
+	switch_mutex_lock(session->frame_read_mutex);
+
 	channel = switch_core_session_get_channel(session);
 
 	switch_channel_set_flag(channel, CF_SERVICE);
@@ -330,7 +338,10 @@ static void *switch_core_service_thread(switch_thread_t *thread, void *obj)
 		}
 	}
 
+	switch_mutex_unlock(session->frame_read_mutex);
+
 	switch_core_session_rwunlock(session);
+
 	return NULL;
 }
 
@@ -342,7 +353,7 @@ SWITCH_DECLARE(void) switch_core_thread_session_end(switch_core_session_t *sessi
 
 	channel = switch_core_session_get_channel(session);
 	switch_assert(channel);
-	
+
 	switch_channel_clear_flag(channel, CF_SERVICE);
 }
 
@@ -353,8 +364,8 @@ SWITCH_DECLARE(void) switch_core_service_session(switch_core_session_t *session)
 
 	channel = switch_core_session_get_channel(session);
 	switch_assert(channel);
-	
-	switch_core_session_launch_thread(session, switch_core_service_thread, session);
+
+	switch_core_session_launch_thread(session, (void *(*)(switch_thread_t *,void *))switch_core_service_thread, session);
 }
 
 /* This function abstracts the thread creation for modules by allowing you to pass a function ptr and
@@ -446,6 +457,30 @@ SWITCH_DECLARE(void) switch_core_set_globals(void)
 #endif
 	}
 
+	if (!SWITCH_GLOBAL_dirs.run_dir && (SWITCH_GLOBAL_dirs.run_dir = (char *) malloc(BUFSIZE))) {
+#ifdef SWITCH_RUN_DIR
+		switch_snprintf(SWITCH_GLOBAL_dirs.run_dir, BUFSIZE, "%s", SWITCH_RUN_DIR);
+#else
+		switch_snprintf(SWITCH_GLOBAL_dirs.run_dir, BUFSIZE, "%s%srun", base_dir, SWITCH_PATH_SEPARATOR);
+#endif
+	}
+
+	if (!SWITCH_GLOBAL_dirs.recordings_dir && (SWITCH_GLOBAL_dirs.recordings_dir = (char *) malloc(BUFSIZE))) {
+#ifdef SWITCH_RECORDINGS_DIR
+		switch_snprintf(SWITCH_GLOBAL_dirs.recordings_dir, BUFSIZE, "%s", SWITCH_RECORDINGS_DIR);
+#else
+		switch_snprintf(SWITCH_GLOBAL_dirs.recordings_dir, BUFSIZE, "%s%srecordings", base_dir, SWITCH_PATH_SEPARATOR);
+#endif
+	}
+
+	if (!SWITCH_GLOBAL_dirs.sounds_dir && (SWITCH_GLOBAL_dirs.sounds_dir = (char *) malloc(BUFSIZE))) {
+#ifdef SWITCH_SOUNDS_DIR
+		switch_snprintf(SWITCH_GLOBAL_dirs.sounds_dir, BUFSIZE, "%s", SWITCH_SOUNDS_DIR);
+#else
+		switch_snprintf(SWITCH_GLOBAL_dirs.sounds_dir, BUFSIZE, "%s%ssounds", base_dir, SWITCH_PATH_SEPARATOR);
+#endif
+	}
+
 	if (!SWITCH_GLOBAL_dirs.storage_dir && (SWITCH_GLOBAL_dirs.storage_dir = (char *) malloc(BUFSIZE))) {
 #ifdef SWITCH_STORAGE_DIR
 		switch_snprintf(SWITCH_GLOBAL_dirs.storage_dir, BUFSIZE, "%s", SWITCH_STORAGE_DIR);
@@ -503,10 +538,13 @@ SWITCH_DECLARE(void) switch_core_set_globals(void)
 	switch_assert(SWITCH_GLOBAL_dirs.mod_dir);
 	switch_assert(SWITCH_GLOBAL_dirs.conf_dir);
 	switch_assert(SWITCH_GLOBAL_dirs.log_dir);
+	switch_assert(SWITCH_GLOBAL_dirs.run_dir);
 	switch_assert(SWITCH_GLOBAL_dirs.db_dir);
 	switch_assert(SWITCH_GLOBAL_dirs.script_dir);
 	switch_assert(SWITCH_GLOBAL_dirs.htdocs_dir);
 	switch_assert(SWITCH_GLOBAL_dirs.grammar_dir);
+	switch_assert(SWITCH_GLOBAL_dirs.recordings_dir);
+	switch_assert(SWITCH_GLOBAL_dirs.sounds_dir);
 	switch_assert(SWITCH_GLOBAL_dirs.temp_dir);
 }
 
@@ -752,7 +790,7 @@ static void load_mime_types(void)
 	if (fd <= 0) {
 		goto end;
 	}
-	
+
 	while ((switch_fd_read_line(fd, line_buf, sizeof(line_buf)))) {
 		char *p;
 		char *type = line_buf;
@@ -782,7 +820,7 @@ static void load_mime_types(void)
 		fd = -1;
 	}
 
- end:
+  end:
 
 	switch_safe_free(mime_path);
 
@@ -854,8 +892,27 @@ SWITCH_DECLARE(switch_bool_t) switch_check_network_list_ip_token(const char *ip_
 	if ((list = switch_core_hash_find(IP_LIST.hash, list_name))) {
 		ok = switch_network_list_validate_ip_token(list, ip, token);
 	} else if (strchr(list_name, '/')) {
-		switch_parse_cidr(list_name, &net, &mask, &bits);
-		ok = switch_test_subnet(ip, net, mask);
+		if (strchr(list_name, ',')) {
+			char *list_name_dup = strdup(list_name);
+			char *argv[32];
+			int argc;
+
+			switch_assert(list_name_dup);
+
+			if ((argc = switch_separate_string(list_name_dup, ',', argv, (sizeof(argv) / sizeof(argv[0]))))) {
+				int i;
+				for (i = 0; i < argc; i++) {
+					switch_parse_cidr(argv[i], &net, &mask, &bits);
+					if ((ok = switch_test_subnet(ip, net, mask))) {
+						break;
+					}
+				}
+			}
+			free(list_name_dup);
+		} else {
+			switch_parse_cidr(list_name, &net, &mask, &bits);
+			ok = switch_test_subnet(ip, net, mask);
+		}
 	}
 	switch_mutex_unlock(runtime.global_mutex);
 
@@ -876,7 +933,7 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 	switch_find_local_ip(guess_ip, sizeof(guess_ip), &mask, AF_INET);
 	in.s_addr = mask;
 	switch_set_string(guess_mask, inet_ntoa(in));
-	
+
 	switch_mutex_lock(runtime.global_mutex);
 
 	if (IP_LIST.hash) {
@@ -898,7 +955,7 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 	switch_network_list_add_cidr(rfc_list, "10.0.0.0/8", SWITCH_TRUE);
 	switch_network_list_add_cidr(rfc_list, "172.16.0.0/12", SWITCH_TRUE);
 	switch_network_list_add_cidr(rfc_list, "192.168.0.0/16", SWITCH_TRUE);
-	switch_core_hash_insert(IP_LIST.hash, tmp_name, rfc_list);	
+	switch_core_hash_insert(IP_LIST.hash, tmp_name, rfc_list);
 
 	tmp_name = "wan.auto";
 	switch_network_list_create(&rfc_list, tmp_name, SWITCH_TRUE, IP_LIST.pool);
@@ -906,19 +963,18 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 	switch_network_list_add_cidr(rfc_list, "10.0.0.0/8", SWITCH_FALSE);
 	switch_network_list_add_cidr(rfc_list, "172.16.0.0/12", SWITCH_FALSE);
 	switch_network_list_add_cidr(rfc_list, "192.168.0.0/16", SWITCH_FALSE);
-	switch_core_hash_insert(IP_LIST.hash, tmp_name, rfc_list);	
+	switch_core_hash_insert(IP_LIST.hash, tmp_name, rfc_list);
 
 	tmp_name = "nat.auto";
 	switch_network_list_create(&rfc_list, tmp_name, SWITCH_FALSE, IP_LIST.pool);
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Created ip list %s default (deny)\n", tmp_name);
 	if (switch_network_list_add_host_mask(rfc_list, guess_ip, guess_mask, SWITCH_FALSE) == SWITCH_STATUS_SUCCESS) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE,
-						  "Adding %s/%s (deny) to list %s\n", guess_ip, guess_mask, tmp_name);
-        }
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Adding %s/%s (deny) to list %s\n", guess_ip, guess_mask, tmp_name);
+	}
 	switch_network_list_add_cidr(rfc_list, "10.0.0.0/8", SWITCH_TRUE);
 	switch_network_list_add_cidr(rfc_list, "172.16.0.0/12", SWITCH_TRUE);
 	switch_network_list_add_cidr(rfc_list, "192.168.0.0/16", SWITCH_TRUE);
-	switch_core_hash_insert(IP_LIST.hash, tmp_name, rfc_list);	
+	switch_core_hash_insert(IP_LIST.hash, tmp_name, rfc_list);
 
 	tmp_name = "loopback.auto";
 	switch_network_list_create(&rfc_list, tmp_name, SWITCH_FALSE, IP_LIST.pool);
@@ -929,14 +985,13 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 	tmp_name = "localnet.auto";
 	switch_network_list_create(&list, tmp_name, SWITCH_FALSE, IP_LIST.pool);
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Created ip list %s default (deny)\n", tmp_name);
-	
-	if (switch_network_list_add_host_mask(list, guess_ip, guess_mask, SWITCH_TRUE) == SWITCH_STATUS_SUCCESS) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE,
-						  "Adding %s/%s (allow) to list %s\n", guess_ip, guess_mask, tmp_name);
-	}
-	switch_core_hash_insert(IP_LIST.hash, tmp_name, list);	
 
-	
+	if (switch_network_list_add_host_mask(list, guess_ip, guess_mask, SWITCH_TRUE) == SWITCH_STATUS_SUCCESS) {
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Adding %s/%s (allow) to list %s\n", guess_ip, guess_mask, tmp_name);
+	}
+	switch_core_hash_insert(IP_LIST.hash, tmp_name, list);
+
+
 	if ((xml = switch_xml_open_cfg("acl.conf", &cfg, NULL))) {
 		if ((x_lists = switch_xml_child(cfg, "network-lists"))) {
 			for (x_list = switch_xml_child(x_lists, "list"); x_list; x_list = x_list->next) {
@@ -944,7 +999,7 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 				const char *dft = switch_xml_attr(x_list, "default");
 				switch_bool_t default_type = SWITCH_TRUE;
 
-				if (switch_strlen_zero(name)) {
+				if (zstr(name)) {
 					continue;
 				}
 
@@ -986,13 +1041,13 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 						switch_assert(my_params);
 						switch_event_add_header_string(my_params, SWITCH_STACK_BOTTOM, "domain", domain);
 						switch_event_add_header_string(my_params, SWITCH_STACK_BOTTOM, "purpose", "network-list");
-						
+
 						if (switch_xml_locate_domain(domain, my_params, &xml_root, &x_domain) != SWITCH_STATUS_SUCCESS) {
 							switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "Cannot locate domain %s\n", domain);
 							switch_event_destroy(&my_params);
 							continue;
 						}
-						
+
 						switch_event_destroy(&my_params);
 
 						for (ut = switch_xml_child(x_domain, "user"); ut; ut = ut->next) {
@@ -1001,25 +1056,8 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 
 							if (id && user_cidr) {
 								char *token = switch_mprintf("%s@%s", id, domain);
-								char *argv[100] = { 0 };
-								char *dup_cidr = strdup(user_cidr);
-								int argc, i = 0;
-								
 								switch_assert(token);
-								switch_assert(dup_cidr);
-
-								if ((argc = switch_separate_string(dup_cidr, ',', argv, (sizeof(argv) / sizeof(argv[0]))))) {
-									for (i = 0; i < argc; i++) {
-										if (switch_network_list_add_cidr_token(list, argv[i], ok, token) == SWITCH_STATUS_SUCCESS) {
-											switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Adding %s (%s) [%s] to list %s\n", 
-															  argv[i], ok ? "allow" : "deny", switch_str_nil(token), name);
-										} else {
-											switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error Adding %s (%s) [%s] to list %s\n",
-															  argv[i], ok ? "allow" : "deny", switch_str_nil(token), name);
-										}
-									}
-								}
-								free(dup_cidr);
+								switch_network_list_add_cidr_token(list, user_cidr, ok, token);
 								free(token);
 							}
 						}
@@ -1030,35 +1068,18 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 									for (ut = switch_xml_child(uts, "user"); ut; ut = ut->next) {
 										const char *user_cidr = switch_xml_attr(ut, "cidr");
 										const char *id = switch_xml_attr(ut, "id");
-										
+
 										if (id && user_cidr) {
 											char *token = switch_mprintf("%s@%s", id, domain);
-											char *argv[100] = { 0 };
-											char *dup_cidr = strdup(user_cidr);
-											int argc, i = 0;
-											
 											switch_assert(token);
-											switch_assert(dup_cidr);
-
-											if ((argc = switch_separate_string(dup_cidr, ',', argv, (sizeof(argv) / sizeof(argv[0]))))) {
-												for (i = 0; i < argc; i++) {
-													if (switch_network_list_add_cidr_token(list, argv[i], ok, token) == SWITCH_STATUS_SUCCESS) {
-														switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Adding %s (%s) [%s] to list %s\n", 
-																		  argv[i], ok ? "allow" : "deny", switch_str_nil(token), name);
-													} else {
-														switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error Adding %s (%s) [%s] to list %s\n",
-																		  argv[i], ok ? "allow" : "deny", switch_str_nil(token), name);
-													}
-												}
-											}
-											free(dup_cidr);
+											switch_network_list_add_cidr_token(list, user_cidr, ok, token);
 											free(token);
 										}
 									}
 								}
 							}
 						}
-						
+
 						switch_xml_free(xml_root);
 					} else if (cidr) {
 						if (switch_network_list_add_cidr(list, cidr, ok) == SWITCH_STATUS_SUCCESS) {
@@ -1072,7 +1093,7 @@ SWITCH_DECLARE(void) switch_load_network_lists(switch_bool_t reload)
 							switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE,
 											  "Adding %s/%s (%s) to list %s\n", host, mask, ok ? "allow" : "deny", name);
 						}
-					} 
+					}
 
 					switch_core_hash_insert(IP_LIST.hash, name, list);
 				}
@@ -1091,6 +1112,9 @@ SWITCH_DECLARE(uint32_t) switch_core_max_dtmf_duration(uint32_t duration)
 		if (duration > SWITCH_MAX_DTMF_DURATION) {
 			duration = SWITCH_MAX_DTMF_DURATION;
 		}
+		if (duration < SWITCH_MIN_DTMF_DURATION) {
+			duration = SWITCH_MIN_DTMF_DURATION;
+		}
 		runtime.max_dtmf_duration = duration;
 	}
 	return runtime.max_dtmf_duration;
@@ -1099,19 +1123,35 @@ SWITCH_DECLARE(uint32_t) switch_core_max_dtmf_duration(uint32_t duration)
 SWITCH_DECLARE(uint32_t) switch_core_default_dtmf_duration(uint32_t duration)
 {
 	if (duration) {
-		if (duration < SWITCH_DEFAULT_DTMF_DURATION) {
-			duration = SWITCH_DEFAULT_DTMF_DURATION;
+		if (duration < SWITCH_MIN_DTMF_DURATION) {
+			duration = SWITCH_MIN_DTMF_DURATION;
+		}
+		if (duration > SWITCH_MAX_DTMF_DURATION) {
+			duration = SWITCH_MAX_DTMF_DURATION;
 		}
 		runtime.default_dtmf_duration = duration;
 	}
 	return runtime.default_dtmf_duration;
 }
 
+SWITCH_DECLARE(uint32_t) switch_core_min_dtmf_duration(uint32_t duration)
+{
+	if (duration) {
+		if (duration < SWITCH_MIN_DTMF_DURATION) {
+			duration = SWITCH_MIN_DTMF_DURATION;
+		}
+		if (duration > SWITCH_MAX_DTMF_DURATION) {
+			duration = SWITCH_MAX_DTMF_DURATION;
+		}
+	}
+	return runtime.min_dtmf_duration;
+}
+
 static void switch_core_set_serial(void)
 {
 	char buf[13] = "";
 	char path[256];
-	
+
 	int fd = -1, write_fd = -1;
 	switch_ssize_t bytes = 0;
 
@@ -1125,9 +1165,9 @@ static void switch_core_set_serial(void)
 		int i = 0;
 
 		switch_inet_pton(AF_INET, ip, &ipi);
-		byte = (switch_byte_t *) &ipi;
+		byte = (switch_byte_t *) & ipi;
 
-		for(i = 0; i < 8; i += 2) {
+		for (i = 0; i < 8; i += 2) {
 			switch_snprintf(buf + i, sizeof(buf) - i, "%0.2x", *byte);
 			byte++;
 		}
@@ -1144,20 +1184,24 @@ static void switch_core_set_serial(void)
 		close(fd);
 		fd = -1;
 	}
-	
+
 	switch_core_set_variable("switch_serial", buf);
 }
 
-
 SWITCH_DECLARE(switch_status_t) switch_core_init(switch_core_flag_t flags, switch_bool_t console, const char **err)
 {
 	switch_uuid_t uuid;
 	char guess_ip[256];
-	char *dir_path;
 	int mask = 0;
 	struct in_addr in;
+	char hostname[256] = "";
 
-	memset(&runtime, 0, sizeof(runtime));
+	if (runtime.runlevel > 0) {
+		/* one per customer */
+		return SWITCH_STATUS_SUCCESS;
+	}
+
+	runtime.runlevel++;
 
 	runtime.dummy_cng_frame.data = runtime.dummy_data;
 	runtime.dummy_cng_frame.datalen = sizeof(runtime.dummy_data);
@@ -1170,6 +1214,7 @@ SWITCH_DECLARE(switch_status_t) switch_core_init(switch_core_flag_t flags, switc
 	runtime.mailer_app_args = "-t";
 	runtime.max_dtmf_duration = SWITCH_MAX_DTMF_DURATION;
 	runtime.default_dtmf_duration = SWITCH_DEFAULT_DTMF_DURATION;
+	runtime.min_dtmf_duration = SWITCH_MIN_DTMF_DURATION;
 
 	/* INIT APR and Create the pool context */
 	if (apr_initialize() != SWITCH_STATUS_SUCCESS) {
@@ -1183,19 +1228,20 @@ SWITCH_DECLARE(switch_status_t) switch_core_init(switch_core_flag_t flags, switc
 	}
 	switch_assert(runtime.memory_pool != NULL);
 
-	dir_path = switch_mprintf("%s%ssounds", SWITCH_GLOBAL_dirs.base_dir, SWITCH_PATH_SEPARATOR);
-	switch_dir_make_recursive(dir_path, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
-	switch_safe_free(dir_path);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.base_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.mod_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.conf_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.log_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
+	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.run_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.db_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.script_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.htdocs_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.grammar_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
+	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.recordings_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
+	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.sounds_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 	switch_dir_make_recursive(SWITCH_GLOBAL_dirs.temp_dir, SWITCH_DEFAULT_DIR_PERMS, runtime.memory_pool);
 
+	switch_mutex_init(&runtime.uuid_mutex, SWITCH_MUTEX_NESTED, runtime.memory_pool);
 
 	switch_mutex_init(&runtime.throttle_mutex, SWITCH_MUTEX_NESTED, runtime.memory_pool);
 	switch_mutex_init(&runtime.session_hash_mutex, SWITCH_MUTEX_NESTED, runtime.memory_pool);
@@ -1215,6 +1261,9 @@ SWITCH_DECLARE(switch_status_t) switch_core_init(switch_core_flag_t flags, switc
 		runtime.console = stdout;
 	}
 
+	gethostname(hostname, sizeof(hostname));
+	switch_core_set_variable("hostname", hostname);
+
 	switch_find_local_ip(guess_ip, sizeof(guess_ip), &mask, AF_INET);
 	switch_core_set_variable("local_ip_v4", guess_ip);
 	in.s_addr = mask;
@@ -1224,8 +1273,12 @@ SWITCH_DECLARE(switch_status_t) switch_core_init(switch_core_flag_t flags, switc
 	switch_find_local_ip(guess_ip, sizeof(guess_ip), NULL, AF_INET6);
 	switch_core_set_variable("local_ip_v6", guess_ip);
 	switch_core_set_variable("base_dir", SWITCH_GLOBAL_dirs.base_dir);
+	switch_core_set_variable("recordings_dir", SWITCH_GLOBAL_dirs.recordings_dir);
+	switch_core_set_variable("sound_prefix", SWITCH_GLOBAL_dirs.sounds_dir);
+	switch_core_set_variable("sounds_dir", SWITCH_GLOBAL_dirs.sounds_dir);
 	switch_core_set_serial();
 
+	switch_console_init(runtime.memory_pool);
 	switch_event_init(runtime.memory_pool);
 
 	if (switch_xml_init(runtime.memory_pool, err) != SWITCH_STATUS_SUCCESS) {
@@ -1236,25 +1289,27 @@ SWITCH_DECLARE(switch_status_t) switch_core_init(switch_core_flag_t flags, switc
 	if (switch_test_flag((&runtime), SCF_USE_AUTO_NAT)) {
 		switch_nat_init(runtime.memory_pool);
 	}
-	
+
 	switch_log_init(runtime.memory_pool, runtime.colorize_console);
 
+	runtime.tipping_point = 5000;
+	runtime.timer_affinity = -1;
 	switch_load_core_config("switch.conf");
 
 
 	switch_core_state_machine_init(runtime.memory_pool);
 
-	if (switch_test_flag((&runtime), SCF_USE_SQL)) {
-		switch_core_sqldb_start(runtime.memory_pool);
+	if (switch_core_sqldb_start(runtime.memory_pool, switch_test_flag((&runtime), SCF_USE_SQL) ? SWITCH_TRUE : SWITCH_FALSE) != SWITCH_STATUS_SUCCESS) {
+		abort();
 	}
+
 	switch_scheduler_task_thread_start();
 
 	switch_rtp_init(runtime.memory_pool);
 
 	runtime.running = 1;
-
 	runtime.initiated = switch_time_now();
-
+	
 	switch_scheduler_add_task(switch_epoch_time_now(NULL), heartbeat_callback, "heartbeat", "core", 0, NULL, SSHF_NONE | SSHF_NO_DEL);
 
 	switch_uuid_get(&uuid);
@@ -1308,13 +1363,6 @@ static void handle_SIGBUS(int sig)
 }
 #endif
 
-/* no ctl-c mofo */
-static void handle_SIGINT(int sig)
-{
-	if (sig);
-	return;
-}
-
 static void handle_SIGHUP(int sig)
 {
 	if (sig) {
@@ -1332,20 +1380,16 @@ static void handle_SIGHUP(int sig)
 static void switch_load_core_config(const char *file)
 {
 	switch_xml_t xml = NULL, cfg = NULL;
-	
+
 	if ((xml = switch_xml_open_cfg(file, &cfg, NULL))) {
 		switch_xml_t settings, param;
-		
+
 		if ((settings = switch_xml_child(cfg, "settings"))) {
 			for (param = switch_xml_child(settings, "param"); param; param = param->next) {
 				const char *var = switch_xml_attr_soft(param, "name");
 				const char *val = switch_xml_attr_soft(param, "value");
 
-				if (!strcasecmp(var, "crash-protection")) {
-					if (switch_true(val)) {
-						switch_set_flag((&runtime), SCF_CRASH_PROT);
-					}
-				} else if (!strcasecmp(var, "loglevel")) {
+				if (!strcasecmp(var, "loglevel")) {
 					int level;
 					if (*val > 47 && *val < 58) {
 						level = atoi(val);
@@ -1364,32 +1408,72 @@ static void switch_load_core_config(const char *file)
 					rlp.rlim_max = RLIM_INFINITY;
 					setrlimit(RLIMIT_CORE, &rlp);
 #endif
+				} else if (!strcasecmp(var, "debug-level")) {
+					int tmp = atoi(val);
+					if (tmp > -1 && tmp < 11) {
+						switch_core_session_ctl(SCSC_DEBUG_LEVEL, &tmp);
+					}
+				} else if (!strcasecmp(var, "enable-early-hangup") && switch_true(val)) {
+					switch_set_flag((&runtime), SCF_EARLY_HANGUP);
 				} else if (!strcasecmp(var, "colorize-console") && switch_true(val)) {
 					runtime.colorize_console = SWITCH_TRUE;
-				} else if (!strcasecmp(var, "mailer-app") && !switch_strlen_zero(val)) {
+				} else if (!strcasecmp(var, "mailer-app") && !zstr(val)) {
 					runtime.mailer_app = switch_core_strdup(runtime.memory_pool, val);
 				} else if (!strcasecmp(var, "mailer-app-args") && val) {
 					runtime.mailer_app_args = switch_core_strdup(runtime.memory_pool, val);
-				} else if (!strcasecmp(var, "sessions-per-second") && !switch_strlen_zero(val)) {
+				} else if (!strcasecmp(var, "sessions-per-second") && !zstr(val)) {
 					switch_core_sessions_per_second(atoi(val));
-				} else if (!strcasecmp(var, "max_dtmf_duration") && !switch_strlen_zero(val)) {
+				} else if (!strcasecmp(var, "max-dtmf-duration") && !zstr(val)) {
 					int tmp = atoi(val);
 					if (tmp > 0) {
 						switch_core_max_dtmf_duration((uint32_t) tmp);
 					}
-				} else if (!strcasecmp(var, "default_dtmf_duration") && !switch_strlen_zero(val)) {
+				} else if (!strcasecmp(var, "min-dtmf-duration") && !zstr(val)) {
+					int tmp = atoi(val);
+					if (tmp > 0) {
+						switch_core_min_dtmf_duration((uint32_t) tmp);
+					}
+				} else if (!strcasecmp(var, "default-dtmf-duration") && !zstr(val)) {
 					int tmp = atoi(val);
 					if (tmp > 0) {
 						switch_core_default_dtmf_duration((uint32_t) tmp);
 					}
-				} else if (!strcasecmp(var, "disable-monotonic-timing")) {
-					switch_time_set_monotonic(SWITCH_FALSE);
-				} else if (!strcasecmp(var, "max-sessions") && !switch_strlen_zero(val)) {
+				} else if (!strcasecmp(var, "enable-monotonic-timing")) {
+					switch_time_set_monotonic(switch_true(var));
+				} else if (!strcasecmp(var, "enable-clock-nanosleep")) {
+					switch_time_set_nanosleep(switch_true(var));
+				} else if (!strcasecmp(var, "enable-cond-yield")) {
+					switch_time_set_cond_yield(switch_true(var));
+				} else if (!strcasecmp(var, "enable-timer-matrix")) {
+					switch_time_set_matrix(switch_true(var));
+				} else if (!strcasecmp(var, "max-sessions") && !zstr(val)) {
 					switch_core_session_limit(atoi(val));
-				} else if (!strcasecmp(var, "rtp-start-port") && !switch_strlen_zero(val)) {
+				} else if (!strcasecmp(var, "min-idle-cpu") && !zstr(val)) {
+					switch_core_min_idle_cpu(atof(val));
+				} else if (!strcasecmp(var, "tipping-point") && !zstr(val)) {
+					runtime.tipping_point = atoi(val);
+				} else if (!strcasecmp(var, "timer-affinity") && !zstr(val)) {
+					if (!strcasecmp(val, "disabled")) {
+						runtime.timer_affinity = -1;
+					} else {
+						runtime.timer_affinity = atoi(val);
+					}
+				} else if (!strcasecmp(var, "rtp-start-port") && !zstr(val)) {
 					switch_rtp_set_start_port((switch_port_t) atoi(val));
-				} else if (!strcasecmp(var, "rtp-end-port") && !switch_strlen_zero(val)) {
+				} else if (!strcasecmp(var, "rtp-end-port") && !zstr(val)) {
 					switch_rtp_set_end_port((switch_port_t) atoi(val));
+				} else if (!strcasecmp(var, "core-db-dsn") && !zstr(val)) {
+					if (switch_odbc_available()) {
+						runtime.odbc_dsn = switch_core_strdup(runtime.memory_pool, val);
+						if ((runtime.odbc_user = strchr(runtime.odbc_dsn, ':'))) {
+							*runtime.odbc_user++ = '\0';
+							if ((runtime.odbc_pass = strchr(runtime.odbc_user, ':'))) {
+								*runtime.odbc_pass++ = '\0';
+							}
+						}
+					} else {
+						switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "ODBC IS NOT AVAILABLE!\n");
+					}
 #ifdef ENABLE_ZRTP
 				} else if (!strcasecmp(var, "rtp-enable-zrtp")) {
 					switch_core_set_variable("zrtp_enabled", val);
@@ -1414,6 +1498,24 @@ static void switch_load_core_config(const char *file)
 
 }
 
+SWITCH_DECLARE(const char *) switch_core_banner(void)
+{
+
+
+	return ("\n"
+			"   _____              ______        _____ _____ ____ _   _  \n"
+			"  |  ___| __ ___  ___/ ___\\ \\      / /_ _|_   _/ ___| | | | \n"
+			"  | |_ | '__/ _ \\/ _ \\___ \\\\ \\ /\\ / / | |  | || |   | |_| | \n"
+			"  |  _|| | |  __/  __/___) |\\ V  V /  | |  | || |___|  _  | \n"
+			"  |_|  |_|  \\___|\\___|____/  \\_/\\_/  |___| |_| \\____|_| |_| \n"
+			"\n"
+			"************************************************************\n"
+			"* Anthony Minessale II, Michael Jerris, Brian West, Others *\n"
+			"* FreeSWITCH (http://www.freeswitch.org)                   *\n"
+			"* Paypal Donations Appreciated: paypal@freeswitch.org      *\n"
+			"* Brought to you by ClueCon http://www.cluecon.com/        *\n" "************************************************************\n" "\n");
+}
+
 
 SWITCH_DECLARE(switch_status_t) switch_core_init_and_modload(switch_core_flag_t flags, switch_bool_t console, const char **err)
 {
@@ -1423,8 +1525,15 @@ SWITCH_DECLARE(switch_status_t) switch_core_init_and_modload(switch_core_flag_t
 		return SWITCH_STATUS_GENERR;
 	}
 
+	if (runtime.runlevel > 1) {
+		/* one per customer */
+		return SWITCH_STATUS_SUCCESS;
+	}
+
+	runtime.runlevel++;
+
 	/* set signal handlers */
-	signal(SIGINT, handle_SIGINT);
+	signal(SIGINT, SIG_IGN);
 #ifdef SIGPIPE
 	signal(SIGPIPE, handle_SIGPIPE);
 #endif
@@ -1463,9 +1572,11 @@ SWITCH_DECLARE(switch_status_t) switch_core_init_and_modload(switch_core_flag_t
 		switch_event_fire(&event);
 	}
 
+	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CONSOLE, "%s", switch_core_banner());
+
+
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CONSOLE,
-					  "\nFreeSWITCH Version %s Started.\nCrash Protection [%s]\nMax Sessions[%u]\nSession Rate[%d]\nSQL [%s]\n", SWITCH_VERSION_FULL,
-					  switch_test_flag((&runtime), SCF_CRASH_PROT) ? "Enabled" : "Disabled",
+					  "\nFreeSWITCH Version %s Started.\nMax Sessions[%u]\nSession Rate[%d]\nSQL [%s]\n", SWITCH_VERSION_FULL,
 					  switch_core_session_limit(0),
 					  switch_core_sessions_per_second(0), switch_test_flag((&runtime), SCF_USE_SQL) ? "Enabled" : "Disabled");
 
@@ -1497,22 +1608,58 @@ SWITCH_DECLARE(switch_time_t) switch_core_uptime(void)
 	return switch_micro_time_now() - runtime.initiated;
 }
 
-SWITCH_DECLARE(int32_t) switch_core_session_ctl(switch_session_ctl_t cmd, int32_t *val)
+
+#ifdef _MSC_VER
+static void win_shutdown(void)
 {
+
+	HANDLE shutdown_event;
+	char path[512];
+	/* for windows we need the event to signal for shutting down a background FreeSWITCH */
+	snprintf(path, sizeof(path), "Global\\Freeswitch.%d", getpid());
+
+	/* open the event so we can signal it */
+	shutdown_event = OpenEvent(EVENT_MODIFY_STATE, FALSE, path);
+
+	if (shutdown_event) {
+		/* signal the event to shutdown */
+		SetEvent(shutdown_event);
+		/* cleanup */
+		CloseHandle(shutdown_event);
+	}
+}
+#endif
+
+SWITCH_DECLARE(uint32_t) switch_core_debug_level(void)
+{
+	return runtime.debug_level;
+}
+
+
+SWITCH_DECLARE(int32_t) switch_core_session_ctl(switch_session_ctl_t cmd, void *val)
+{
+	int *intval = (int *) val;
+
 	if (switch_test_flag((&runtime), SCF_SHUTTING_DOWN)) {
 		return -1;
 	}
 
 	switch (cmd) {
+	case SCSC_CALIBRATE_CLOCK:
+		switch_time_calibrate_clock();
+		break;
+	case SCSC_FLUSH_DB_HANDLES:
+		switch_cache_db_flush_handles();
+		break;
 	case SCSC_SEND_SIGHUP:
 		handle_SIGHUP(1);
 		break;
 	case SCSC_SYNC_CLOCK:
 		switch_time_sync();
-		*val = 0;
+		*intval = 0;
 		break;
 	case SCSC_PAUSE_INBOUND:
-		if (*val) {
+		if (*intval) {
 			switch_set_flag((&runtime), SCF_NO_NEW_SESSIONS);
 		} else {
 			switch_clear_flag((&runtime), SCF_NO_NEW_SESSIONS);
@@ -1524,6 +1671,18 @@ SWITCH_DECLARE(int32_t) switch_core_session_ctl(switch_session_ctl_t cmd, int32_
 	case SCSC_CANCEL_SHUTDOWN:
 		switch_clear_flag((&runtime), SCF_SHUTDOWN_REQUESTED);
 		break;
+	case SCSC_SAVE_HISTORY:
+		switch_console_save_history();
+		break;
+	case SCSC_CRASH:
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Declinatio Mortuus Obfirmo!\n");
+		switch_console_save_history();
+		abort();
+		break;
+	case SCSC_SHUTDOWN_NOW:
+		switch_console_save_history();
+		exit(0);
+		break;
 	case SCSC_SHUTDOWN_ELEGANT:
 	case SCSC_SHUTDOWN_ASAP:
 		{
@@ -1535,24 +1694,30 @@ SWITCH_DECLARE(int32_t) switch_core_session_ctl(switch_session_ctl_t cmd, int32_
 				switch_set_flag((&runtime), SCF_NO_NEW_SESSIONS);
 			}
 
-			while(runtime.running && switch_test_flag((&runtime), SCF_SHUTDOWN_REQUESTED) && (count = switch_core_session_count())) {
+			while (runtime.running && switch_test_flag((&runtime), SCF_SHUTDOWN_REQUESTED) && (count = switch_core_session_count())) {
 				switch_yield(500000);
 				if (++x == 20) {
-					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, 
-									  "Shutdown in progress, %u session(s) remain.\nShutting down %s\n", 
-									  count,
-									  cmd == SCSC_SHUTDOWN_ASAP ? "ASAP" : "Once there are no active calls.");
+					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING,
+									  "Shutdown in progress, %u session(s) remain.\nShutting down %s\n",
+									  count, cmd == SCSC_SHUTDOWN_ASAP ? "ASAP" : "once there are no active calls.");
 					x = 0;
 				}
 			}
-			
+
 			if (switch_test_flag((&runtime), SCF_SHUTDOWN_REQUESTED)) {
 				switch_set_flag((&runtime), SCF_NO_NEW_SESSIONS);
-				if (*val) {
+#ifdef _MSC_VER
+				win_shutdown();
+#endif
+
+				if (*intval) {
 					switch_set_flag((&runtime), SCF_RESTART);
 					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "Restarting\n");
 				} else {
 					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "Shutting down\n");
+#ifdef _MSC_VER
+					fclose(stdin);
+#endif
 				}
 				runtime.running = 0;
 			} else {
@@ -1562,51 +1727,76 @@ SWITCH_DECLARE(int32_t) switch_core_session_ctl(switch_session_ctl_t cmd, int32_
 		}
 		break;
 	case SCSC_SHUTDOWN:
-		if (*val) {
+
+#ifdef _MSC_VER
+		win_shutdown();
+#endif
+
+		if (*intval) {
 			switch_set_flag((&runtime), SCF_RESTART);
 			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "Restarting\n");
 		} else {
 			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_INFO, "Shutting down\n");
+#ifdef _MSC_VER
+			fclose(stdin);
+#endif
 		}
 		runtime.running = 0;
 		break;
 	case SCSC_CHECK_RUNNING:
-		*val = runtime.running;
+		*intval = runtime.running;
 		break;
 	case SCSC_LOGLEVEL:
-		if (*val > -1) {
-			runtime.hard_log_level = *val;
+		if (*intval > -1) {
+			runtime.hard_log_level = *intval;
 		}
 
 		if (runtime.hard_log_level > SWITCH_LOG_DEBUG) {
 			runtime.hard_log_level = SWITCH_LOG_DEBUG;
 		}
-		*val = runtime.hard_log_level;
+		*intval = runtime.hard_log_level;
+		break;
+	case SCSC_DEBUG_LEVEL:
+		if (*intval > -1) {
+			if (*intval > 10)
+				*intval = 10;
+			runtime.debug_level = *intval;
+		}
+		*intval = runtime.debug_level;
+		break;
+	case SCSC_MIN_IDLE_CPU:
+		{
+			double *dval = (double *) val;
+			*dval = switch_core_min_idle_cpu(*dval);
+		}
 		break;
 	case SCSC_MAX_SESSIONS:
-		*val = switch_core_session_limit(*val);
+		*intval = switch_core_session_limit(*intval);
 		break;
 	case SCSC_LAST_SPS:
-		*val = runtime.sps_last;
+		*intval = runtime.sps_last;
 		break;
 	case SCSC_MAX_DTMF_DURATION:
-		*val = switch_core_max_dtmf_duration(*val);
+		*intval = switch_core_max_dtmf_duration(*intval);
+		break;
+	case SCSC_MIN_DTMF_DURATION:
+		*intval = switch_core_min_dtmf_duration(*intval);
 		break;
 	case SCSC_DEFAULT_DTMF_DURATION:
-		*val = switch_core_default_dtmf_duration(*val);
+		*intval = switch_core_default_dtmf_duration(*intval);
 		break;
 	case SCSC_SPS:
 		switch_mutex_lock(runtime.throttle_mutex);
-		if (*val > 0) {
-			runtime.sps_total = *val;
+		if (*intval > 0) {
+			runtime.sps_total = *intval;
 		}
-		*val = runtime.sps_total;
+		*intval = runtime.sps_total;
 		switch_mutex_unlock(runtime.throttle_mutex);
 		break;
 
 	case SCSC_RECLAIM:
 		switch_core_memory_reclaim_all();
-		*val = 0;
+		*intval = 0;
 		break;
 	}
 
@@ -1635,24 +1825,26 @@ SWITCH_DECLARE(switch_status_t) switch_core_destroy(void)
 
 	switch_set_flag((&runtime), SCF_NO_NEW_SESSIONS);
 	switch_set_flag((&runtime), SCF_SHUTTING_DOWN);
-	
+
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CONSOLE, "End existing sessions\n");
 	switch_core_session_hupall(SWITCH_CAUSE_SYSTEM_SHUTDOWN);
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CONSOLE, "Clean up modules.\n");
 
 	switch_loadable_module_shutdown();
-	
+
 	if (switch_test_flag((&runtime), SCF_USE_SQL)) {
 		switch_core_sqldb_stop();
 	}
 	switch_scheduler_task_thread_stop();
-	
+
 	switch_rtp_shutdown();
 	if (switch_test_flag((&runtime), SCF_USE_AUTO_NAT)) {
 		switch_nat_shutdown();
 	}
 	switch_xml_destroy();
 
+	switch_console_shutdown();
+
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CONSOLE, "Closing Event Engine.\n");
 	switch_event_shutdown();
 
@@ -1675,6 +1867,8 @@ SWITCH_DECLARE(switch_status_t) switch_core_destroy(void)
 	switch_safe_free(SWITCH_GLOBAL_dirs.script_dir);
 	switch_safe_free(SWITCH_GLOBAL_dirs.htdocs_dir);
 	switch_safe_free(SWITCH_GLOBAL_dirs.grammar_dir);
+	switch_safe_free(SWITCH_GLOBAL_dirs.recordings_dir);
+	switch_safe_free(SWITCH_GLOBAL_dirs.sounds_dir);
 	switch_safe_free(SWITCH_GLOBAL_dirs.temp_dir);
 
 	switch_core_hash_destroy(&runtime.global_vars);
@@ -1684,11 +1878,11 @@ SWITCH_DECLARE(switch_status_t) switch_core_destroy(void)
 		apr_pool_destroy(runtime.memory_pool);
 		apr_terminate();
 	}
-	
+
 	return switch_test_flag((&runtime), SCF_RESTART) ? SWITCH_STATUS_RESTART : SWITCH_STATUS_SUCCESS;
 }
 
-SWITCH_DECLARE(switch_status_t) switch_core_chat_send(const char *name, const char *proto, const char *from, const char *to, 
+SWITCH_DECLARE(switch_status_t) switch_core_chat_send(const char *name, const char *proto, const char *from, const char *to,
 													  const char *subject, const char *body, const char *type, const char *hint)
 {
 	switch_chat_interface_t *ci;
@@ -1727,18 +1921,18 @@ SWITCH_DECLARE(void) switch_core_memory_reclaim_all(void)
 
 
 struct system_thread_handle {
-	const char * cmd;
+	const char *cmd;
 	switch_thread_cond_t *cond;
 	switch_mutex_t *mutex;
 	switch_memory_pool_t *pool;
 	int ret;
 };
 
-static void *SWITCH_THREAD_FUNC system_thread(switch_thread_t *thread, void *obj) 
+static void *SWITCH_THREAD_FUNC system_thread(switch_thread_t *thread, void *obj)
 {
-	struct system_thread_handle *sth = (struct system_thread_handle *)obj;
+	struct system_thread_handle *sth = (struct system_thread_handle *) obj;
 
-#if 0 // if we are a luser we can never turn this back down, didn't we already set the stack size?
+#if 0							// if we are a luser we can never turn this back down, didn't we already set the stack size?
 #if defined(HAVE_SETRLIMIT) && !defined(__FreeBSD__)
 	struct rlimit rlim;
 
@@ -1755,11 +1949,10 @@ static void *SWITCH_THREAD_FUNC system_thread(switch_thread_t *thread, void *obj
 #if 0
 #if defined(HAVE_SETRLIMIT) && !defined(__FreeBSD__)
 	rlim.rlim_cur = SWITCH_THREAD_STACKSIZE;
-	rlim.rlim_max = SWITCH_SYSTEM_THREAD_STACKSIZE; 
+	rlim.rlim_max = SWITCH_SYSTEM_THREAD_STACKSIZE;
 	if (setrlimit(RLIMIT_STACK, &rlim) < 0) {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Setting stack size failed! (%s)\n", strerror(errno));
 	}
-
 #endif
 #endif
 
